#include "OTA.h"
#include "../../Static.h"

ArduinoOTAUpdater::ArduinoOTAUpdater() : BufferedInputSource("ArduinoOTA") {
  ArduinoOTA.onStart(&ArduinoOTAUpdater::s_onStart);
  ArduinoOTA.onProgress(&ArduinoOTAUpdater::s_onProgress);
  ArduinoOTA.onError(&ArduinoOTAUpdater::s_onError);
  ArduinoOTA.onEnd(&ArduinoOTAUpdater::s_onFinished);
  ArduinoOTA.setRebootOnSuccess(false);
}

void ArduinoOTAUpdater::loop() {
    if (m_online && !m_updating) {
        ArduinoOTA.handle();
    }
    BufferedInputSource::loop();
}

void ArduinoOTAUpdater::handleEvent(const InputEvent& evt) {
    if (evt.intent == InputEvent::NetworkStatus && evt.asInt()) {
        Log.notice("Booting OTA");
        m_online = true;
        ArduinoOTA.begin();
    }
}

void ArduinoOTAUpdater::s_onFinished()
{
  Log.notice("OTA complete!");
  Static<ArduinoOTAUpdater>::instance()->m_updating = false;
  Platform::restart();
}

void ArduinoOTAUpdater::s_onStart() {
    Log.notice("OTA Start!");
    Static<ArduinoOTAUpdater>::instance()->m_updating = true;
    Static<ArduinoOTAUpdater>::instance()->setEvent(InputEvent::FirmwareUpdate);
}

void ArduinoOTAUpdater::s_onError(ota_error_t err)
{
  Log.notice("OTA failure: %d", (int)err);
  Static<ArduinoOTAUpdater>::instance()->m_updating = false;
}

void ArduinoOTAUpdater::s_onProgress(unsigned int progress, unsigned int total) {
    Log.notice("OTA Progress! %d / %d", progress, total);
    Static<ArduinoOTAUpdater>::instance()->setEvent(InputEvent{InputEvent::FirmwareUpdate, progress});
    // Try to run the main loop in case we're not in a threaded updater
    // environment. This keeps rendering running, and pets the watchdog.
    MainLoop::instance()->loop();
}

STATIC_ALLOC(ArduinoOTAUpdater);
STATIC_TASK(ArduinoOTAUpdater);
